function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from "react";
import { useState, forwardRef, useRef, useMemo } from "react";
import PropTypes from "prop-types";
import { Editor, EditorState, ContentState } from "draft-js";
import { createDecorator } from "./createDecorator.js";
import { Selection } from "./Selection.js";
var HighlightWithinTextarea = forwardRef(function (props, fwdRef) {
  var highlight = props.highlight,
      onChange = props.onChange;
  var value = props.value,
      selection = props.selection;

  var _useState = useState(),
      forceUpdate = _useState[1];

  var ref = useRef({});
  var editorState;
  var _ref$current = ref.current,
      prevValue = _ref$current.prevValue,
      prevEditorState = _ref$current.prevEditorState,
      nextValue = _ref$current.nextValue,
      nextEditorState = _ref$current.nextEditorState;

  if (nextValue == value) {
    // Change was accepted.
    editorState = nextEditorState;
  } else if (prevValue == value) {
    // They blocked the state change.
    editorState = prevEditorState;

    if (!selection && nextValue) {
      selection = new Selection(editorState);
      selection.focus = Math.max(selection.anchor, selection.focus);
      selection.anchor = selection.focus;
    }
  } else if (prevEditorState) {
    // They chose a whole new value.
    var _contentState = ContentState.createFromText(value);

    var changeType = "change-block-data";
    editorState = EditorState.push(prevEditorState, _contentState, changeType);

    if (!selection) {
      var fixedValue, offset;

      if (nextEditorState) {
        selection = new Selection(nextEditorState);
        fixedValue = value.replaceAll("\r\n", "\n");
        offset = fixedValue.length - nextValue.length;
      } else {
        selection = new Selection(prevEditorState);
        fixedValue = value.replaceAll("\r\n", "\n");
        offset = fixedValue.length - prevValue.length;
      }

      selection.anchor += offset;
      selection.focus += offset;
    }
  } else {
    // First time in here.
    var _contentState2 = ContentState.createFromText(value);

    editorState = EditorState.createWithContent(_contentState2);
  }

  var contentState = editorState.getCurrentContent();
  var decorator = useMemo(function () {
    return createDecorator(contentState, highlight, value);
  }, [contentState, highlight, value]);
  editorState = EditorState.set(editorState, {
    decorator: decorator
  });

  if (selection) {
    editorState = selection.forceSelection(editorState);
  }

  ref.current = {
    prevEditorState: editorState,
    prevValue: value
  };

  var onDraftChange = function onDraftChange(nextEditorState) {
    var nextValue = nextEditorState.getCurrentContent().getPlainText();
    ref.current = _extends({}, ref.current, {
      nextEditorState: nextEditorState,
      nextValue: nextValue
    });
    var selection = undefined;
    selection = new Selection(nextEditorState);
    onChange(nextValue, selection);
    forceUpdate({});
  };

  var newProps = _extends({}, props);

  delete newProps.highlight;
  delete newProps.selection;
  delete newProps.value;
  delete newProps.onChange;
  return /*#__PURE__*/React.createElement(Editor, _extends({
    editorState: editorState,
    onChange: onDraftChange
  }, newProps, {
    ref: fwdRef
  }));
});
HighlightWithinTextarea.propTypes = process.env.NODE_ENV !== "production" ? {
  onChange: PropTypes.func,
  value: PropTypes.string.isRequired,
  highlight: PropTypes.oneOfType([PropTypes.string, PropTypes.array, PropTypes.instanceOf(RegExp), PropTypes.object, PropTypes.func]),
  placeholder: PropTypes.string,
  selection: PropTypes.instanceOf(Selection),
  textAlignment: PropTypes.string,
  textDirectionality: PropTypes.string,
  autoCapitalize: PropTypes.string,
  autoComplete: PropTypes.string,
  autoCorrect: PropTypes.string,
  readOnly: PropTypes.bool,
  spellCheck: PropTypes.bool,
  stripPastedStyles: PropTypes.bool,
  editorKey: PropTypes.string,
  handleReturn: PropTypes.func,
  handleKeyCommand: PropTypes.func,
  handleBeforeInput: PropTypes.func,
  handlePastedText: PropTypes.func,
  handlePastedFiles: PropTypes.func,
  handleDroppedFiles: PropTypes.func,
  handleDrop: PropTypes.func,
  keyBindingFn: PropTypes.func,
  onFocus: PropTypes.func,
  onBlur: PropTypes.func
} : {};
HighlightWithinTextarea.defaultProps = {
  highlight: null,
  placeholder: "Enter some text..."
};
export default HighlightWithinTextarea;