"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _draftJs = require("draft-js");

var _createDecorator = require("./createDecorator.js");

var _Selection = require("./Selection.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var HighlightWithinTextarea = (0, _react.forwardRef)(function (props, fwdRef) {
  var highlight = props.highlight,
      onChange = props.onChange;
  var value = props.value,
      selection = props.selection;

  var _useState = (0, _react.useState)(),
      forceUpdate = _useState[1];

  var ref = (0, _react.useRef)({});
  var editorState;
  var _ref$current = ref.current,
      prevValue = _ref$current.prevValue,
      prevEditorState = _ref$current.prevEditorState,
      nextValue = _ref$current.nextValue,
      nextEditorState = _ref$current.nextEditorState;

  if (nextValue == value) {
    // Change was accepted.
    editorState = nextEditorState;
  } else if (prevValue == value) {
    // They blocked the state change.
    editorState = prevEditorState;

    if (!selection && nextValue) {
      selection = new _Selection.Selection(editorState);
      selection.focus = Math.max(selection.anchor, selection.focus);
      selection.anchor = selection.focus;
    }
  } else if (prevEditorState) {
    // They chose a whole new value.
    var _contentState = _draftJs.ContentState.createFromText(value);

    var changeType = "change-block-data";
    editorState = _draftJs.EditorState.push(prevEditorState, _contentState, changeType);

    if (!selection) {
      var fixedValue, offset;

      if (nextEditorState) {
        selection = new _Selection.Selection(nextEditorState);
        fixedValue = value.replaceAll("\r\n", "\n");
        offset = fixedValue.length - nextValue.length;
      } else {
        selection = new _Selection.Selection(prevEditorState);
        fixedValue = value.replaceAll("\r\n", "\n");
        offset = fixedValue.length - prevValue.length;
      }

      selection.anchor += offset;
      selection.focus += offset;
    }
  } else {
    // First time in here.
    var _contentState2 = _draftJs.ContentState.createFromText(value);

    editorState = _draftJs.EditorState.createWithContent(_contentState2);
  }

  var contentState = editorState.getCurrentContent();
  var decorator = (0, _react.useMemo)(function () {
    return (0, _createDecorator.createDecorator)(contentState, highlight, value);
  }, [contentState, highlight, value]);
  editorState = _draftJs.EditorState.set(editorState, {
    decorator: decorator
  });

  if (selection) {
    editorState = selection.forceSelection(editorState);
  }

  ref.current = {
    prevEditorState: editorState,
    prevValue: value
  };

  var onDraftChange = function onDraftChange(nextEditorState) {
    var nextValue = nextEditorState.getCurrentContent().getPlainText();
    ref.current = _extends({}, ref.current, {
      nextEditorState: nextEditorState,
      nextValue: nextValue
    });
    var selection = undefined;
    selection = new _Selection.Selection(nextEditorState);
    onChange(nextValue, selection);
    forceUpdate({});
  };

  var newProps = _extends({}, props);

  delete newProps.highlight;
  delete newProps.selection;
  delete newProps.value;
  delete newProps.onChange;
  return /*#__PURE__*/_react["default"].createElement(_draftJs.Editor, _extends({
    editorState: editorState,
    onChange: onDraftChange
  }, newProps, {
    ref: fwdRef
  }));
});
HighlightWithinTextarea.propTypes = process.env.NODE_ENV !== "production" ? {
  onChange: _propTypes["default"].func,
  value: _propTypes["default"].string.isRequired,
  highlight: _propTypes["default"].oneOfType([_propTypes["default"].string, _propTypes["default"].array, _propTypes["default"].instanceOf(RegExp), _propTypes["default"].object, _propTypes["default"].func]),
  placeholder: _propTypes["default"].string,
  selection: _propTypes["default"].instanceOf(_Selection.Selection),
  textAlignment: _propTypes["default"].string,
  textDirectionality: _propTypes["default"].string,
  autoCapitalize: _propTypes["default"].string,
  autoComplete: _propTypes["default"].string,
  autoCorrect: _propTypes["default"].string,
  readOnly: _propTypes["default"].bool,
  spellCheck: _propTypes["default"].bool,
  stripPastedStyles: _propTypes["default"].bool,
  editorKey: _propTypes["default"].string,
  handleReturn: _propTypes["default"].func,
  handleKeyCommand: _propTypes["default"].func,
  handleBeforeInput: _propTypes["default"].func,
  handlePastedText: _propTypes["default"].func,
  handlePastedFiles: _propTypes["default"].func,
  handleDroppedFiles: _propTypes["default"].func,
  handleDrop: _propTypes["default"].func,
  keyBindingFn: _propTypes["default"].func,
  onFocus: _propTypes["default"].func,
  onBlur: _propTypes["default"].func
} : {};
HighlightWithinTextarea.defaultProps = {
  highlight: null,
  placeholder: "Enter some text..."
};
var _default = HighlightWithinTextarea;
exports["default"] = _default;
module.exports = exports.default;